// CQGCELEventHandlers.cpp : implementation file
//

#include "stdafx.h"
#include "Common.h"
#include "Helper.h"
#include "ConstantVolumeBars.h"
#include "ConstantVolumeBarsDlg.h"

STDMETHODIMP CConstantVolumeBarsDlg::OnDataError(
                           LPDISPATCH /*obj*/,
                           BSTR errorDescription)
{
   CString errMsg("Data error: ");
   errMsg += ATL::CW2A(errorDescription);
   AfxMessageBox(errMsg);

   return S_OK;
}

STDMETHODIMP CConstantVolumeBarsDlg::OnDataConnectionStatusChanged(
                           eConnectionStatus newStatus)
{
   try
   {
      m_lblDataConnection.SetWindowText(_T("Data ") + CHelper::ConnectionStatusToString(newStatus));

      EnableControls();
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CConstantVolumeBarsDlg::OnConstantVolumeBarsResolved(
                           ICQGConstantVolumeBars* cqg_constant_volume_bars,
                           ICQGError* /*cqg_error*/)
{
   try
   {
      if (!m_spConstantVolumeBars)
      {
         return S_OK;
      }

      // Get selected constant volume bars ID
      ATL::CComBSTR selConstantVolumeBarsID;
      HRESULT hr = m_spConstantVolumeBars->get_Id(&selConstantVolumeBarsID);
      Common::ThrowOnCOMError(m_spConstantVolumeBars, hr);

      // Get changed constant volume bars ID
      ATL::CComBSTR constantVolumeBarsID;
      hr = cqg_constant_volume_bars->get_Id(&constantVolumeBarsID);
      Common::ThrowOnCOMError<ICQGConstantVolumeBars>(cqg_constant_volume_bars, hr);

      // Check if constant volume bars resolved in the selected request
      if (selConstantVolumeBarsID == constantVolumeBarsID)
      {
         ClearAllData();
         ShowAllData();

         EnableControls();
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CConstantVolumeBarsDlg::OnConstantVolumeBarsAdded(
                           ICQGConstantVolumeBars* cqg_constant_volume_bars)
{
   try
   {
      if (!m_spConstantVolumeBars)
      {
         return S_OK;
      }

      // Get selected constant volume bars ID
      ATL::CComBSTR selConstantVolumeBarsID;
      HRESULT hr = m_spConstantVolumeBars->get_Id(&selConstantVolumeBarsID);
      Common::ThrowOnCOMError(m_spConstantVolumeBars, hr);

      // Get changed constant volume bars ID
      ATL::CComBSTR constantVolumeBarsID;
      hr = cqg_constant_volume_bars->get_Id(&constantVolumeBarsID);
      Common::ThrowOnCOMError<ICQGConstantVolumeBars>(cqg_constant_volume_bars, hr);

      // Check if record added in the selected request
      if (selConstantVolumeBarsID == constantVolumeBarsID)
      {
         // Show changed data
         ShowChangedData();
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CConstantVolumeBarsDlg::OnConstantVolumeBarsInserted(
                           ICQGConstantVolumeBars* cqg_constant_volume_bars,
                           long /*index_*/)
{
   try
   {
      if (!m_spConstantVolumeBars)
      {
         return S_OK;
      }

      // Get selected constant volume bars ID
      ATL::CComBSTR selConstantVolumeBarsID;
      HRESULT hr = m_spConstantVolumeBars->get_Id(&selConstantVolumeBarsID);
      Common::ThrowOnCOMError(m_spConstantVolumeBars, hr);

      // Get changed constant volume bars ID
      ATL::CComBSTR constantVolumeBarsID;
      hr = cqg_constant_volume_bars->get_Id(&constantVolumeBarsID);
      Common::ThrowOnCOMError<ICQGConstantVolumeBars>(cqg_constant_volume_bars, hr);

      // Check if record inserted in the selected request
      if (selConstantVolumeBarsID == constantVolumeBarsID)
      {
         // Show changed data
         ShowChangedData();
      }
   }
   catch(const std::exception & e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CConstantVolumeBarsDlg::OnConstantVolumeBarsRemoved(
                           ICQGConstantVolumeBars* cqg_constant_volume_bars,
                           long /*index_*/)
{
   try
   {
      if (!m_spConstantVolumeBars)
      {
         return S_OK;
      }

      // Get selected constant volume bars ID
      ATL::CComBSTR selConstantVolumeBarsID;
      HRESULT hr = m_spConstantVolumeBars->get_Id(&selConstantVolumeBarsID);
      Common::ThrowOnCOMError(m_spConstantVolumeBars, hr);

      // Get changed constant volume bars ID
      ATL::CComBSTR constantVolumeBarsID;
      hr = cqg_constant_volume_bars->get_Id(&constantVolumeBarsID);
      Common::ThrowOnCOMError<ICQGConstantVolumeBars>(cqg_constant_volume_bars, hr);

      if (selConstantVolumeBarsID == constantVolumeBarsID)
      {
         // Show changed data
         ShowChangedData();
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
   return S_OK;
}

STDMETHODIMP CConstantVolumeBarsDlg::OnConstantVolumeBarsUpdated(
                           ICQGConstantVolumeBars* cqg_constant_volume_bars,
                           long index_)
{
   try
   {
      if (!m_spConstantVolumeBars)
      {
         return S_OK;
      }

      // Get selected constant volume bars ID
      ATL::CComBSTR selConstantVolumeBarsID;
      HRESULT hr = m_spConstantVolumeBars->get_Id(&selConstantVolumeBarsID);
      Common::ThrowOnCOMError(m_spConstantVolumeBars, hr);

      // Get changed constant volume bars ID
      ATL::CComBSTR constantVolumeBarsID;
      hr = cqg_constant_volume_bars->get_Id(&constantVolumeBarsID);
      Common::ThrowOnCOMError<ICQGConstantVolumeBars>(cqg_constant_volume_bars, hr);

      // Check if record updated in the selected request
      if (selConstantVolumeBarsID == constantVolumeBarsID)
      {
         // Find row for specified constant volume bar
         CString indexStr;
         indexStr.Format(_T("%d"), index_ + 1);
         LVFINDINFO findInfo;
         findInfo.flags = LVFI_STRING;
         findInfo.psz = indexStr;
         int row = m_lvLastConstantVolumeBars.FindItem(&findInfo);

         // If found change it's properties
         if (row > -1)
         {
            ShowRecord(index_ + 1, m_lvLastConstantVolumeBars, row);
         }
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}