// Common.cpp : implementation file
//

#include "stdafx.h"
#include "Common.h"

CString Common::DateTimeToString(
                        DATE dateTime, 
                        bool withMilliseconds)
{
   const COleDateTime oleDateTime(dateTime);

   if (withMilliseconds)
   {
      CString dateTimeStr = oleDateTime.Format(_T("%d/%m/%Y"));

      const int MILLISECONDS_IN_SECOND = 1000;
      const int SECONDS_IN_MINUTES = 60;
      const int MINUTES_IN_HOUR = 60;
      const int HOURS_IN_DAY = 24;
      const int MILLISECONDS_IN_DAY = HOURS_IN_DAY * MINUTES_IN_HOUR * SECONDS_IN_MINUTES * MILLISECONDS_IN_SECOND;

      const int datePart = static_cast<int>(dateTime);
      const double timePart = (dateTime > 0) ? dateTime - datePart : datePart - dateTime;
      const int totalMillisecondsInDay = static_cast<int>(timePart * MILLISECONDS_IN_DAY + 0.5);

      const int milliseconds = totalMillisecondsInDay % MILLISECONDS_IN_SECOND;
      int quotient = totalMillisecondsInDay / MILLISECONDS_IN_SECOND;
      const int seconds = quotient % SECONDS_IN_MINUTES;
      quotient /= SECONDS_IN_MINUTES;
      const int minutes = quotient % MINUTES_IN_HOUR;
      quotient /= MINUTES_IN_HOUR;
      const int hours = quotient % HOURS_IN_DAY;

      dateTimeStr.AppendFormat(_T(" %02d:%02d:%02d.%03d"), hours, minutes, seconds, milliseconds);
      return dateTimeStr;
   }
   else
   {
      return oleDateTime.Format(_T("%d/%m/%Y %H:%M"));
   }
}

CString Common::VariantToString(
                        const ATL::CComVariant& value, bool withMilliseconds /*= false*/)
{
   CString strValue;
   switch (value.vt)
   {
   case VT_I4:
      strValue.Format(_T("%d"), value.intVal);
      break;
   case VT_UI4:
      strValue.Format(_T("%d"), value.uintVal);
      break;
   case VT_R8:
      strValue.Format(_T("%.2f"), value.dblVal);
      break;
   case VT_DATE:
      strValue = DateTimeToString(value.date, withMilliseconds);
      break;
   case VT_BSTR:
      strValue = value.bstrVal;
      break;
   case VT_BOOL:
      strValue = (value.boolVal == VARIANT_TRUE) ? "True" : "False";
      break;
   default:
      throw std::invalid_argument("Unknown variant type");
   }

   return strValue;
}
