/*
* =====================================================================
*  Filename: ConstantVolumeBarsDlg.h
*    Author: Sevada Gyurjyan
*   Company: CQG, Inc.
* Copyright: (c)2008 by CQG,Inc. All rights reserved. 
* =====================================================================
*/

#pragma once

#include "afxwin.h"
#include "afxcmn.h"
#include "afxdtctl.h"
#include "NumEdit.h"

class CConstantVolumeBarsDlg;

/**
\typedef ATL::IDispEventImpl<1, CConstantVolumeBarsDlg, 
                              &__uuidof(_ICQGCELEvents), 
                              &__uuidof(__CQG), 4, 0> ICQGCELDispEventImpl
\brief  Convenient definition of IDispEventImpl instantiation
*/
typedef ATL::IDispEventImpl<1,
                              CConstantVolumeBarsDlg,
                              &__uuidof(_ICQGCELEvents),
                              &__uuidof(__CQG), 4, 0
                           >
                           ICQGCELDispEventImpl;

// CConstantVolumeBarsDlg dialog
class CConstantVolumeBarsDlg : public CDialog, public ICQGCELDispEventImpl
{

public:

   /// \brief CConstantVolumeBarsDlg constructor.
   /// \param pParent - parent window
   explicit CConstantVolumeBarsDlg(
                  CWnd* pParent = NULL); 

   /// \enum Dialog Data
   enum { IDD = IDD_CONSTANT_VOLUME_BARS_DIALOG };

private:

   /// \brief Dialog icon.
   HICON m_hIcon;

   /// \brief DDX/DDV support.
   virtual void DoDataExchange(
                  CDataExchange* pDX);

   /// \brief This message is sent to the dialog box immediately before the dialog box is displayed.
   /// \return Specifies whether the framework shall set the input 
   /// \return focus to the first control of the dialog box
   virtual BOOL OnInitDialog();

   /// \brief Override to prevent dialog from closing on Enter
   /// \brief and catch Enters in tag name and tag value edit controls.
   virtual void OnOK();

private:

   /// \brief Prepares the container for painting.
   afx_msg void OnPaint();

   /// \brief The system calls this function to obtain the cursor to
   /// \brief display while the user drags the minimized window.
   afx_msg HCURSOR OnQueryDragIcon();

   /// \brief Used to unadvise from CQGCEL events and shutdown it.
   afx_msg void OnDestroy();

   /// \brief Changes selected constant volume bars object, and shows its data.
   afx_msg void OnCbnSelChangeCmbConstantVolumeBars();

   /// \brief Opens request settings form to fill up request parameters.
   afx_msg void OnBnClickedBtnRequest();

   /// \brief Removes selected constant volume bars from CQGCEL and combo box control.
   afx_msg void OnBnClickedBtnRemove();

   /// \brief Removes all constant volume bars from CQGCEL and combo box control.
   afx_msg void OnBnClickedBtnRemoveAll();

   /// \brief Used to increment or decrement start range value for filtering.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnDeltaposSpnStart(
                  NMHDR *pNMHDR,
                  LRESULT *pResult);

   /// \brief Used to increment or decrement end range value for filtering.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnDeltaposSpnEnd(
                  NMHDR *pNMHDR,
                  LRESULT *pResult);

   /// \brief Shows filtered constant volume bars.
   afx_msg void OnBnClickedBtnShow();

   /// \brief Declares that the class defines a message map.
   DECLARE_MESSAGE_MAP()

public:

   /// \brief This map is used to declare handler function for specified event.
   /// \brief You can use "OLE/COM Object Viewer" to open CQGCEL-4_0.dll and get event ids.
   /// \brief The handler signature as well as the event ids can be found in the type library
   /// \brief or shown by "OLE/COM Object Viewer".
   BEGIN_SINK_MAP(CConstantVolumeBarsDlg)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x0000000a, OnDataError)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x00000003, OnDataConnectionStatusChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x0000001b, OnConstantVolumeBarsResolved)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x0000001c, OnConstantVolumeBarsAdded)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x0000001d, OnConstantVolumeBarsUpdated)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x0000003b, OnConstantVolumeBarsInserted)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x0000003c, OnConstantVolumeBarsRemoved)
   END_SINK_MAP()

private:

   /// \brief Handle event of some abnormal discrepancy
   /// \brief between data expected and data received from CQG.
   /// \brief This event also can be fired when CQGCEL startup fails.
   /// \param obj - Object where the error occurred
   /// \param errorDescription - string describing the error
   /// \return S_OK
   STDMETHOD(OnDataError)(
                  LPDISPATCH obj,
                  BSTR errorDescription);

   /// \brief Fired when some changes occurred in the connection with the CQG data server.
   /// \param newStatus - new connection status
   /// \return S_OK
   STDMETHOD(OnDataConnectionStatusChanged)(
                  eConnectionStatus newStatus);

   /// \brief Fired when the collection of constant volume bars (CQGConstantVolumeBars) is resolved
   /// \brief or when some error occurred during constant volume bars request processing
   /// \param cqg_constant_volume_bars - reference to resolved CQGConstantVolumeBars
   /// \param cqg_error - CQGError object describing last error occurred during processing the request.
   /// \return S_OK
   STDMETHOD(OnConstantVolumeBarsResolved)(
                  ICQGConstantVolumeBars* cqg_constant_volume_bars,
                  ICQGError* cqg_error);

   /// \brief Fired when CQGConstantVolumeBar item is added to the end of CQGConstantVolumeBars.
   /// \param cqg_constant_volume_bars - reference to changed CQGConstantVolumeBars
   /// \return S_OK
   STDMETHOD(OnConstantVolumeBarsAdded)(
                  ICQGConstantVolumeBars* cqg_constant_volume_bars);

   /// \brief Fired when CQGConstantVolumeBar item is updated.
   /// \param cqg_constant_volume_bars - reference to changed CQGConstantVolumeBars
   /// \param index_ - specifies the updated CQGConstantVolumeBar index
   /// \return S_OK
   STDMETHOD(OnConstantVolumeBarsUpdated)(
                  ICQGConstantVolumeBars* cqg_constant_volume_bars,
                  long index_);

   /// \brief Fired when CQGConstantVolumeBar item is inserted.
   /// \param cqg_constant_volume_bars - reference to changed CQGConstantVolumeBars
   /// \param index_ - specifies the inserted CQGConstantVolumeBar index
   /// \return S_OK 
   STDMETHOD(OnConstantVolumeBarsInserted)(
                  ICQGConstantVolumeBars* cqg_constant_volume_bars,
                  long index_);

   /// \brief Fired when CQGConstantVolumeBar item is removed.
   /// \param cqg_constant_volume_bars - reference to changed CQGConstantVolumeBars
   /// \param index_ - specifies the removed CQGConstantVolumeBar index
   /// \return S_OK
   STDMETHOD(OnConstantVolumeBarsRemoved)(
                  ICQGConstantVolumeBars* cqg_constant_volume_bars,
                  long index_);

private:

   /// \enum eConstantVolumeBarssColumn. Represents constant volume bars columns in the list.
   enum eRecordsColumns;

private:

   /// \brief Initializes CQGCEL object, and starts the CQGCEL.
   void InitializeCQGCEL();

   /// \brief Initializes all controls.
   void InitializeControls();

   /// \brief Initializes specified list control's columns.
   /// \param listCtrl - list control which columns will be configured
   static void InitializeList(
                  CListCtrl& listCtrl);

   /// \brief Clears all data(parameters\outputs\records) from the form.
   void ClearAllData();

   /// \brief Clears request parameters from the form.
   void ClearRequestParams();

   /// \brief Clears output parameters from the form.
   void ClearOutputParams();
   
   /// \brief Clears request tags.
   void ClearTags();

   /// \brief Clears all records from the specified list.
   /// \param listCtrl - list control which will be cleared
   static void ClearRecords(
                  CListCtrl& listCtrl);

   /// \brief Shows all data(parameters\outputs\records) connected
   /// \brief with selected request.
   void ShowAllData();

   /// \brief Shows given range records in the specified list control, after changes.
   void ShowChangedData();

   /// \brief Shows selected request parameters on the form.
   void ShowRequestParams();

   /// \brief Shows selected request outputs on the form.
   void ShowOutputParams();

   /// \brief Shows given range records in the specified list control.
   /// \param startIndex - range start index
   /// \param endIndex - range end index
   /// \param listCtrl - list control in which records will be shown
   void ShowRecords(
                  long startIndex,
                  long endIndex,
                  CListCtrl& listCtrl);

   /// \brief Adds record with given index into the end of specified list control.
   /// \param recordIndex - record index which will be added
   /// \param listCtrl - list control in which record will be added
   void AddRecord(
                  long recordIndex,
                  CListCtrl& listCtrl);

   /// \brief Shows record with given index into the specified row of the specified list control.
   /// \param recordIndex - record index which will be shown
   /// \param listCtrl - list control in which record will be shown
   /// \param row - row of the list control in which record will be shown
   void ShowRecord(
                  long recordIndex,
                  CListCtrl& listCtrl,
                  long row);

   /// \brief Shows specified property to specified list control's specified position.
   /// \param spConstantVolumeBar - constant volume bar which property will be shown
   /// \param get_Property - property name which will be shown
   /// \param listCtrl - list control in which property will be shown
   /// \param row - row
   /// \param col - column
   template <typename T>
   void ShowProperty(
                  const ATL::CComPtr<ICQGConstantVolumeBar>& spConstantVolumeBar,
                  HRESULT (__stdcall ICQGConstantVolumeBar::*get_Property)(T*),
                  CListCtrl& listCtrl,
                  int row,
                  eRecordsColumns col);

   /// \brief Gets start and end indices and validates them to show records.
   /// \param startIndex - range start index
   /// \param endIndex - range end index
   /// \return Returns true if range is valid.
   bool GetIndices(
                  long& startIndex,
                  long& endIndex);

   /// \brief Clears dates and indices, which are used for filtering.
   void ClearIndices();

   /// \brief Fills indices for filtering.
   void FillIndices();

   /// \brief Used for getting CQGCEL properties values string representation.
   /// \brief If specified value is invalid(see CQGCEL.IsValid() method) then "N/A" is returned.
   /// \param value - variant value
   /// \param valueType - variant type
   /// \param withMilliseconds - add milliseconds to the string, only for VT_DATE
   /// \return String representation of the value.
   CString GetValueAsString(
                  ATL::CComVariant value,
                  VARENUM valueType = VT_EMPTY,
                  bool withMilliseconds = false) const;

   /// \brief Enables/disables controls on the form based on connection status and other parameters.
   void EnableControls();

   /// \brief Shows tag value for specified name.
   void OnEnterEdTagName();

   /// \brief Sets tag value for specified name.
   void OnEnterEdTagValue();

private:

   /// \brief CQGCEL object.
   ATL::CComPtr<ICQGCEL> m_spCQGCEL;

   /// \brief Selected constant volume bars object.
   ATL::CComPtr<ICQGConstantVolumeBars> m_spConstantVolumeBars;

private:

   /// \brief Constant volume bars count that will be shown in the last chunk list.
   static const long m_ShowCount;

   /// \brief Max indices interval for filtering.
   static const long m_MaxIdInterval;

private:

   /// \brief Active constant volume bars ComboBox control
   CComboBox m_cmbConstantVolumeBars;

   /// \brief Data connection indicator Label control.
   CStatic m_lblDataConnection;

   /// \brief Request constant volume bars Button control.
   CButton m_btnRequest;

   /// \brief Remove selected constant volume bars Button control.
   CButton m_btnRemove;

   /// \brief Remove all constant volume bars Button control.
   CButton m_btnRemoveAll;

   /// \brief Last error Label control.
   CStatic m_lblError;

   /// \brief Request status Label control.
   CStatic m_lblStatus;

   /// \brief Show filtered constant volume bars Button control.
   CButton m_btnShow;

   /// \brief Tag name EditBox control.
   CEdit m_edTagName;

   /// \brief Tag value EditBox control.
   CEdit m_edTagValue;

   /// \brief All records count Label control.
   CStatic m_lblAllRecsCount;

   /// \brief Filtered records count Label control.
   CStatic m_lblFltrRecsCount;

   /// \brief Last chunk of constant volume bars ListView control.
   CListCtrl m_lvLastConstantVolumeBars;

   /// \brief Filtering by indices range start EditBox control.
   CNumEdit m_edStart;

   /// \brief Filtering by indices range start SpinButton control.
   CSpinButtonCtrl m_spnStart;

   /// \brief Filtering by indices range end EditBox control.
   CNumEdit m_edEnd;

   /// \brief Filtering by indices range end SpinButton control.
   CSpinButtonCtrl m_spnEnd;

   /// \brief Filtered constant volume bars ListView control.
   CListCtrl m_lvConstantVolumeBars;

   /// \brief String which describes request symbol.
   CString m_Symbol;

   /// \brief String which describes request range start.
   CString m_RangeStart;

   /// \brief String which describes request range end.
   CString m_RangeEnd;

   /// \brief String which describes request updates enabled.
   CString m_UpdatesEnabled;

   /// \brief String which describes request include end.
   CString m_IncludeEnd;

   /// \brief String which describes request volume type.
   CString m_VolumeType;

   /// \brief String which describes request volume level.
   CString m_VolumeLevel;

   /// \brief String which describes request include flat ticks.
   CString m_IncludeFlatTicks;

   /// \brief String which describes request sessions filter.
   CString m_SessionsFilter;

   /// \brief String which describes request continuation.
   CString m_Continuation;

   /// \brief String which describes request equalize closes.
   CString m_EqualizeCloses;

   /// \brief String which describes request days before expiration.
   CString m_DaysBeforeExpiration;

   /// \brief String which describes request session flags.
   CString m_SessionFlags;

   /// \brief String which describes output range start.
   CString m_OutputStart;

   /// \brief String which describes output range end.
   CString m_OutputEnd;
   
   /// \brief String which describes subscription level.
   CString m_SubscriptionLevel;
   
   /// \brief String which describes recalc period.
   CString m_RecalcPeriod;
};

enum CConstantVolumeBarsDlg::eRecordsColumns
{
   rcIndex,
   rcTimestamp,
   rcOpen,
   rcHigh,
   rcLow,
   rcClose,
   rcMid,
   rcHLC3,
   rcAvg,
   rcTrueHigh,
   rcTrueLow,
   rcRange,
   rcTrueRange,
   rcActualVolume,
   rcTickVolume,
   rcAskVolume,
   rcBidVolume
};