/*
* =====================================================================
*  Filename: ConstantVolumeBarsReqDlg.h
*    Author: Sevada Gyurjyan
*   Company: CQG, Inc.
* Copyright: (c)2008 by CQG,Inc. All rights reserved. 
* =====================================================================
*/

#pragma once

#include "afxwin.h"
#include "afxdtctl.h"
#include "NumEdit.h"

// CConstantVolumeBarsReqDlg dialog
class CConstantVolumeBarsReqDlg : public CDialog
{

public:

   /// \brief CConstantVolumeBarsReqDlg constructor.
   /// \param spRequest - ICQGConstantVolumeBarsRequest object
   /// \param lineTime - line time from CQGCEL
   /// \param pParent - parent window
   CConstantVolumeBarsReqDlg(
               const ATL::CComPtr<ICQGConstantVolumeBarsRequest>& spRequest,
               const DATE lineTime,
               CWnd* pParent = NULL);

   /// \enum Dialog Data.
   enum { IDD = IDD_CONSTANT_VOLUME_BARS_REQ_DIALOG };

private:

   /// \brief DDX/DDV support.
   virtual void DoDataExchange(
               CDataExchange* pDX);

   /// \brief This message is sent to the dialog box immediately before the dialog box is displayed.
   /// \return Specifies whether the framework shall set the input 
   /// \return focus to the first control of the dialog box
   virtual BOOL OnInitDialog();

private:

   /// \brief Sets request parameters and closes the form.
   afx_msg void OnBnClickedOk();

   /// \brief Used to increment or decrement integer range start.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnDeltaposSpnRangeStart(
               NMHDR *pNMHDR,
               LRESULT *pResult);

   /// \brief Used to increment or decrement integer range end.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnDeltaposSpnRangeEnd(
               NMHDR *pNMHDR,
               LRESULT *pResult);

   /// \brief Used to increment or decrement volume level.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnDeltaposSpnVolumeLevel(
               NMHDR *pNMHDR,
               LRESULT *pResult);

   /// \brief Used to increment or decrement days before expiration.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnDeltaposSpnDaysBeforeExp(
               NMHDR *pNMHDR,
               LRESULT *pResult);

   /// \brief Used to increment or decrement sessions filter.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnDeltaposSpnSessionsFilter(
               NMHDR *pNMHDR,
               LRESULT *pResult);

   /// \brief Changes request type and unhides/hides relative controls.
   afx_msg void OnBnClickedRdRequestType();

   /// \brief Checks/unchecks custom session and unhides/hides relative controls.
   afx_msg void OnBnClickedChkCustomSession();

   /// \brief Changes continuation type and enables/disables relative controls.
   afx_msg void OnCbnSelChangeCmbContinuation();

   /// \brief Changes subscription level and enables/disables relative controls.
   afx_msg void OnCbnSelChangeCmbSubscription();
   
   /// \brief Used to increment or decrement recalc period.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnDeltaposSpnRecalcPeriod(
               NMHDR *pNMHDR,
               LRESULT *pResult);

   /// \brief Declares that the class defines a message map.
   DECLARE_MESSAGE_MAP()

private:

   /// \brief Initializes all controls.
   void InitializeControls();

   /// \brief Sets all parameters default values taken from CQGConstantVolumeBarsRequest.
   void SetDefaultParameters();

   /// \brief Initializes volume type combo box.
   void InitializeVolumeType();

   /// \brief Initializes continuation types combo box.
   void InitializeContinuationCombo();
   
   /// \brief Initializes subscription combo box.
   void InitializeSubscriptionCombo();
   
   /// \brief Changes request type.
   void RequestTypeChanged();

   /// \brief Checks fields in the form according to specified session flags bit mask.
   /// \param sessionFlag - sessions flag bit mask
   void SetSessionFlags(
               eSessionFlag sessionFlag);

   /// \brief Returns session flag bit mask according to checked fields.
   /// \return Session flag as eSessionFlag object.
   eSessionFlag GetSessionFlags();

private:

   /// \enum eRequestType. Represents request type.
   enum eRequestType;

private:

   /// \brief CQGConstantVolumeBarsRequest object.
   ATL::CComPtr<ICQGConstantVolumeBarsRequest> m_spRequest;

   /// \brief Contains line time from CQGCEL when form activated.
   const DATE m_LineTime;

private:

   /// \brief Request's symbol
   CString m_Symbol;

   /// \brief Request's range start date DateTimePicker control
   CDateTimeCtrl m_dtpRangeStart;

   /// \brief Request's range start date
   COleDateTime m_RangeStartDate;

   /// \brief Request's range start integer EditBox control
   CNumEdit m_edRangeStart;

   /// \brief Request's range start SpinButton control
   CSpinButtonCtrl m_spnRangeStart;

   /// \brief Request's range end date DateTimePicker control
   CDateTimeCtrl m_dtpRangeEnd;

   /// \brief Request's range end date
   COleDateTime m_RangeEndDate;

   /// \brief Request's range end date EditBox control
   CNumEdit m_edRangeEnd;

   /// \brief Request's range end SpinButton control
   CSpinButtonCtrl m_spnRangeEnd;

   /// \brief Request's type
   int m_RequestType;

   /// \brief Request's include end CheckBox control
   CButton m_chkIncludeEnd;

   /// \brief Request's include end
   BOOL m_IncludeEnd;

   /// \brief Request's volume type ComboBox control
   CComboBox m_cmbVolumeType;

   /// \brief Request's volume level Edit control
   CNumEdit m_edVolumelevel;

   /// \brief Request's volume level SpinButton control
   CSpinButtonCtrl m_spnVolumelevel;

   /// \brief Request's include flat ticks
   BOOL m_IncludeFlatTicks;

   /// \brief Request's continuation type ComboBox control
   CComboBox m_cmbContinuation;

   /// \brief Request's equalize closes type CheckBox control
   CButton m_chkEqualizeCloses;

   /// \brief Request's equalize closes
   BOOL m_EqualizeCloses;

   /// \brief Request's days before expiration EditBox control
   CNumEdit m_edDaysBeforeExp;

   /// \brief Request's days before expiration SpinButton control
   CSpinButtonCtrl m_spnDaysBeforeExp;

   /// \brief Request's updates enabled
   BOOL m_UpdatesEnabled;

   /// \brief Request's daily from intraday
   BOOL m_DailyFromIntraday;

   /// \brief Request's sessions filter EditBox control
   CNumEdit m_edSessionsFilter;

   /// \brief Request's sessions filter SpinButton control
   CSpinButtonCtrl m_spnSessionsFilter;

   /// \brief Request's custom sessions filter
   CString m_CustomSessionsFilter;

   /// \brief Request's custom sessions filter EditBox control
   CEdit m_edCustomSessionsFilter;

   /// \brief Request's custom session
   BOOL m_CustomSession;
   
   /// \brief Request's subscription level ComboBox control.
   CComboBox m_cmbSubscription;

   /// \brief Request's recalc period Label control.
   CStatic m_lblRecalcPeriod;

   /// \brief Request's recalc period EditBox control.
   CNumEdit m_edRecalcPeriod;

   /// \brief Request's recalc period SpinButton control.
   CSpinButtonCtrl m_spnRecalcPeriod;
};

enum CConstantVolumeBarsReqDlg::eRequestType
{
   rtIntInt,
   rtDateInt,
   rtDateDate
};